//
//  BFRecordScreenController.swift
//  BFRecordScreenKit_Example
//
//  Created by 胡志强 on 2021/11/23.
//  Copyright © 2021 CocoaPods. All rights reserved.
//

import BFCommonKit
import BFMediaKit
import BFNetRequestKit
import BFUIKit
import Foundation
import GPUImage
import Photos
import UIKit

struct WithDrawModel {
    var type: Int // 0:拖动； 1:预览播放暂停 2: 录音结束  3: 删除录音  4: 删除字幕
    var timestamp: Double
    var deletedVoices: [(PQVoiceModel, Int)]?
    // add by ak 保存删除的字幕数据用于恢复
    var deletedTittles: [(PQEditSubTitleModel, Int)]?
}

public class BFRecordScreenController: BFBaseViewController {
    public var nextActionHandle: (() -> Void)?
    public var closeActionHandle: (() -> Void)?
    public var changeItemHandle: ((_ index: Int) -> Void)?
    // 当前录制结束的上报
    public var recordRndHandle: ((_ currentRecord: PQVoiceModel?) -> Void)?
    // 字幕按钮点击上报
    public var subTitleBtnClickHandle: ((_ isOn: Bool) -> Void)?
    
    // MARK: - 录制参数
    
    public var assets = [PHAsset]()
    var currItemModelIndex = 0
    public var itemModels = [BFRecordItemModel]()
    // add by ak 当前的显示的字幕位置
    var showSubtitleIndex = 0
    //    var shouldPlayRecordIndex:Int = -1          // 当前应该播放的录音资源序号
    var currentPlayRecordIndex: Int = -1 // >= 0 :当前正在播放的录音资源序号； -3： 刚录音完，不需要播放录音； -1:初始化阶段
    public var displayLink: CADisplayLink? // 图片素材播放控制
    var isRecording = false { // 是否正在录音
        didSet {
            withDrawBtn.isHidden = isRecording
            changeVoiceBtn.isHidden = isRecording
            subtitleBtn.isHidden = isRecording
            soundSettingBtn.isHidden = isRecording
            
            recordBtn.setTitle(isRecording ? "松手 暂停" : "按住 录音", for: .normal)
            recordBtn.backgroundColor = UIColor.hexColor(hexadecimal: "#389AFF", alpha: isRecording ? 0.6 : 1)
            playBtn.isSelected = isRecording
//            if !isRecording {
//                BFLog(1, message: "stop")
//            }
        }
    }
    
    var isEndPlay = false {
        didSet {
            recordBtn.isHidden = isEndPlay
        }
    }
    
    var isNormalPlaying = false { // 是否正在播放
        didSet {
            playBtn.isSelected = isNormalPlaying
        }
    }
    
    var currentAssetProgress: CMTime = .zero // 当前素材播放的进度
    // 播放器开始播放时间
    var recordStartPlayTime: CMTime = .zero
    // 某个录音开始播放时间
    var currenStartPlayTime: CMTime = .zero
    var recordStartTime: Double = 0 // 录制开始时间
    var pauseTime: Double = 0 // 停止无操作的时间点
    
    var assetPlayer: AVPlayer? // 原视频音频播放器
    
    var hadPrepareToPlayRecord = false // 录音播放器准备
    var recordPlayer: AVPlayer? // 录音音频播放器
    var movie: GPUImageMovie? // 视频预览
    var playView: GPUImageView? // 视频展示视图
    
    // MARK: 行为参数
    
    var movieIsProcessing = false
    var events = [WithDrawModel]() // 行为记录，方便撤销
    var isDragingProgressSlder: Bool = false // 是否在拖动进度条
    var isStopAtRecordRange = -1
    
    // 保存识别出来的字幕信息,用于回放，和合成使用
    //    var saveSubTitles:[PQEditSubTitleModel] = Array.init()
    
    // MARK: - 录音相关
    
    // 定义音频的编码参数
    let recordSettings: [String: Any] = [AVSampleRateKey: 44100.0, // 声音采样率
                                           AVFormatIDKey: kAudioFormatLinearPCM, // 编码格式
                                   AVNumberOfChannelsKey: 1, // 采集音轨
                                AVEncoderBitDepthHintKey: 16, // 位深
                                AVEncoderAudioQualityKey: AVAudioQuality.medium.rawValue] // 音频质量
    
    public var haveSpeakVolume: Float = 0.0
    public var noSpeakVolume: Float = 1.0
    
    // 录音管理器
    var recorderManager: BFVoiceRecordManager?
    
    // MARK: - 视图参数
    
    var beginOnStartBtn: Bool = false
    var touchStart: CGPoint = CGPoint(x: 0, y: 0)
    var avplayerTimeObserver: NSKeyValueObservation?
    
    var indirectionView: BFIndirectionProgressView?
    
    lazy var progreddL: UILabel = {
        let l = UILabel()
        l.textAlignment = .center
        l.font = UIFont.systemFont(ofSize: 13)
        l.textColor = .white
        l.shadowColor = .black
        l.shadowOffset = CGSize(width: 1, height: 1)
        l.text = "00:00"
        return l
    }()
    
    lazy var playBtn: UIButton = {
        let btn = UIButton(frame: view.bounds)
        btn.setImage(imageInRecordScreenKit(by: "preview_play"), for: .normal)
        let vv = UIView(frame: CGRect(x: 0, y: 0, width: 1, height: 1))
        btn.setImage(vv.graphicsGetImage(), for: .selected)
        btn.addTarget(self, action: #selector(playVideo(btn:)), for: .touchUpInside)
        return btn
    }()
    
    lazy var bottomeView: UIImageView = {
        let iv = UIImageView(image: imageInRecordScreenKit(by: "bottom_shadow"))
        iv.contentMode = .scaleAspectFill
        iv.isUserInteractionEnabled = true
        // 拖曳手勢
        let pan = UIPanGestureRecognizer(target: self, action: #selector(pan(recognizer:)))
        pan.delegate = self
        pan.minimumNumberOfTouches = 1
        pan.maximumNumberOfTouches = 1
        iv.addGestureRecognizer(pan)
        
        return iv
    }()
    
    lazy var recordBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.backgroundColor = ThemeStyleColor
        btn.setTitle("按住 录音", for: .normal)
        btn.addCorner(corner: 6)
        btn.adjustsImageWhenHighlighted = false
        btn.addTarget(self, action: #selector(startRecord), for: .touchDown)
        btn.addTarget(self, action: #selector(endRecord), for: .touchUpInside)
        
        return btn
    }()
    
    lazy var deleteRecordBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.backgroundColor = .red
        btn.setTitle("删除录制", for: .normal)
        btn.adjustsImageWhenHighlighted = false
        btn.addCorner(corner: 6)
        btn.addTarget(self, action: #selector(deleteRecorded), for: .touchUpInside)
        btn.isHidden = true
        return btn
    }()
    
    lazy var withDrawBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "withdraw_n"), for: .normal)
        btn.setImage(imageInRecordScreenKit(by: "withdraw_h"), for: .highlighted)
        btn.setTitle("回退", for: .normal)
        btn.setTitleColor(.white, for: .normal)
        btn.setTitleColor(.gray, for: .highlighted)
        btn.titleLabel?.font = UIFont.systemFont(ofSize: 12)
        btn.contentVerticalAlignment = UIControl.ContentVerticalAlignment.center
        btn.addTarget(self, action: #selector(withdrawAction), for: .touchUpInside)
        return btn
    }()
    
    lazy var changeVoiceBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "changeVoice_n"), for: .normal)
        btn.setImage(imageInRecordScreenKit(by: "changeVoice_h"), for: .highlighted)
        btn.setTitle("变声", for: .normal)
        btn.setTitleColor(.white, for: .normal)
        btn.setTitleColor(ThemeStyleColor, for: .highlighted)
        btn.titleLabel?.font = UIFont.systemFont(ofSize: 12)
        btn.contentVerticalAlignment = UIControl.ContentVerticalAlignment.center
        btn.addTarget(self, action: #selector(changeVoiceAction), for: .touchUpInside)
        return btn
    }()
    
    lazy var toolV: BFIntroduceToolView = {
        let toolV = BFIntroduceToolView()
        toolV.centerY = view.centerY
        
        toolV.choosedToolHandle = { [weak self] tool in
            guard let sself = self else {
                return
            }
            tool.center = sself.view.center
            sself.view.addSubview(tool)
        }
        
        return toolV
        
    }()
    
    // 头像  add by ak
    lazy var avatarView: BFRecordAvatarView = {
        let avatarView = BFRecordAvatarView(frame: CGRect(x: 10, y: 10, width: 120, height: 120))
        avatarView.isHidden = true
        return avatarView
    }()
    
    // 打开摄像头
    lazy var openCameraBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "openCamera"), for: .normal)
        btn.addTarget(self, action: #selector(openCamera), for: .touchUpInside)
        return btn
    }()
    
    // 画笔
    lazy var drawPinBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "drawPin"), for: .normal)
        btn.addTarget(self, action: #selector(drawPin), for: .touchUpInside)
        return btn
    }()
    
    // 字幕设置
    lazy var subtitleBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "subtitleBtn"), for: .normal)
        btn.addTarget(self, action: #selector(subTitleClick), for: .touchUpInside)
        return btn
    }()
    
    // 声音设置
    lazy var soundSettingBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "soundBtn"), for: .normal)
        btn.addTarget(self, action: #selector(soundSetting), for: .touchUpInside)
        return btn
    }()
    
    // 字幕设置面板
    lazy var subtitleSettingView: BFSubtitleSettingView = {
        let subtitleSetting = BFSubtitleSettingView(frame: CGRect(x: 0, y: 0, width: cScreenWidth, height: cScreenHeigth))
        subtitleSetting.isHidden = true
        return subtitleSetting
    }()
    
    // 编辑字幕
    lazy var subtitleEditView: BFSubtitleEditView = {
        let subtitleEditView = BFSubtitleEditView(frame: CGRect(x: 0, y: 0, width: cScreenWidth, height: cScreenHeigth))
        subtitleEditView.isHidden = true
        
        return subtitleEditView
    }()
    
    // 显示字幕
    lazy var subtitleLabel: PQSubTitleLabel = {
        let subtitleLabel = PQSubTitleLabel()
        subtitleLabel.textAlignment = .center
        subtitleLabel.numberOfLines = 0
        subtitleLabel.lineBreakMode = .byWordWrapping
        subtitleLabel.isUserInteractionEnabled = true
        subtitleLabel.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(editSubtitle)))
        return subtitleLabel
        
    }()
    
    // 音量设置
    lazy var audioSettingView: BFAudioSettingView = {
        let audioSettingView = BFAudioSettingView(frame: CGRect(x: 0, y: 0, width: cScreenWidth, height: cScreenHeigth))
        audioSettingView.isHidden = true
        audioSettingView.haveSpeakSlider.valueIsInt = true
        audioSettingView.noSpeakSlider.valueIsInt = true
        audioSettingView.callBack = { [weak self] haveSpeak, noHaveSpeak in
            BFLog(1, message: "haveSpeak is:\(haveSpeak),noHaveSpeak is:\(noHaveSpeak)")
            self?.haveSpeakVolume = haveSpeak / 100.0
            self?.noSpeakVolume = noHaveSpeak / 100.0
        }
        
        return audioSettingView
    }()
    
    lazy var progressThumV: BFVideoThumbProgressView = {
        let vv = BFVideoThumbProgressView(frame: CGRect(x: 0, y: 54, width: cScreenWidth, height: 50))
        vv.dragStartHandle = { [weak self] in
            self?.isDragingProgressSlder = true
            self?.pause()
        }
        vv.dragScrollProgressHandle = { [weak self] isStart, process in
            DispatchQueue.main.async { [weak self] in
                guard let sself = self else {
                    return
                }
                if isStart {
                    sself.events.append(WithDrawModel(type: 0, timestamp: sself.currentAssetProgress.seconds))
                }
                
                sself.isDragingProgressSlder = true
                                
                sself.changeProgress(progress: process)
            }
        }
        vv.dragEndHandle = { [weak self] process in
            DispatchQueue.main.async { [weak self] in
                guard let sself = self else {
                    return
                }
                sself.thumbViewEnded(progress: process)
            }
        }
        vv.isHidden = true
        return vv
    }()
    
    lazy var collectionView: UICollectionView = {
        let flowLayout = UICollectionViewFlowLayout()
        flowLayout.minimumLineSpacing = 0
        flowLayout.minimumInteritemSpacing = 0
        flowLayout.scrollDirection = .horizontal
        flowLayout.itemSize = CGSize(width: view.frame.width, height: view.frame.height)
        let collectionView = UICollectionView(frame: CGRect(x: 0, y: 0, width: view.frame.width, height: view.frame.height), collectionViewLayout: flowLayout)
        collectionView.register(BFImageCoverViewCell.self, forCellWithReuseIdentifier: String(describing: BFImageCoverViewCell.self))
        collectionView.register(BFVideoCoverViewCell.self, forCellWithReuseIdentifier: String(describing: BFVideoCoverViewCell.self))
        collectionView.isPagingEnabled = true
        collectionView.bounces = false
        collectionView.showsVerticalScrollIndicator = false
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.delegate = self
        collectionView.dataSource = self
        collectionView.backgroundColor = UIColor.clear
        if #available(iOS 11.0, *) {
            collectionView.contentInsetAdjustmentBehavior = .never
        } else {
            automaticallyAdjustsScrollViewInsets = false
        }
        return collectionView
    }()
    
    // MARK: - ----------------- 生命周期
    
    deinit {
        cleanMovieTarget()
        NotificationCenter.default.removeObserver(self)
        avplayerTimeObserver?.invalidate()
        if isRecording {
            recorderManager?.stopRecord(isCancel: true)
        }
        assetPlayer?.pause()
        recordPlayer?.pause()
    }
    
    override public func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        navigationController?.isNavigationBarHidden = true
        hiddenNavigation()
    }
    
    override public func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        pause()
    }
    
    override public func viewDidLoad() {
        super.viewDidLoad()
        _ = disablePopGesture()
        
        checkStatus()
        // mdf by ak 切换外放和请求权限分开 否则会导致首次安装时不能执行切换操作
        try? AVAudioSession.sharedInstance().setCategory(.playAndRecord, options: .defaultToSpeaker)
        
        // add by ak 取 nsl token
        BFRecordScreenViewModel.getNlsAccessToken { [weak self] token, appkey in
            BFLog(message: "nls appkey is \(appkey), token is \(token)")
            
            self?.recorderManager = BFVoiceRecordManager(token: token, appid: appkey)
            
            // 录音进度
            self?.recorderManager?.recorderProgrossHandle = { [weak self] progress in
                BFLog(1, message: "curr:录音进度--\(progress) \(self?.recordStartTime ?? 0) \(self?.isRecording ?? false)")
                self?.drawProgressIndication(progress: progress ?? 0)
            }
            
            //录音字幕回调
            self?.recorderManager?.subtitleRecordHandle = {[weak self] asrResult ,audioFilePath in
                if(asrResult == nil){
                    BFLog(message: "识别结果为空？？？？不能生成字幕数据")
                    return
                }
                let dicResult: [String: Any]? = jsonStringToDictionary(asrResult!)
                
                let header = dicResult?["header"] as? [String: Any]
                let payload = dicResult?["payload"] as? [String: Any]
                
                BFLog(message: "识别结果:) \((payload?["result"])!) startTime:\(self?.recorderManager?.voiceModel?.startTime ?? 0.0)")
                DispatchQueue.main.async {
                    // 1，保存字幕数据 begin_time是开始出现文字的时间，time 是结束文字出现的时间 单位都为毫秒，都是相对于录制音频数据整段时间。self.recorderManager.voiceModel?.startTime 为开始的录制的时间，开始和结束都要加上这个时差
                    
                    let newSubtitle = PQEditSubTitleModel()
                    //任务全局唯一ID，请记录该值，便于排查问题。 每次 startRecorder 和 stopRecoder 之间  task_Id都不会变化
                    newSubtitle.taskID = (header?["task_id"] as? String) ?? ""
                    
                    //这里加300ms 是因为返回结果为了切到字，时长提前一些时间，具体时间官方没说和原音频有关系。这里我们先延后300ms 单位：毫秒。
                    
                    newSubtitle.timelineIn = (self?.recorderManager?.voiceModel?.startTime ?? 0.0) + Float64((((payload?["begin_time"]) as? Int) ?? 0) + 300) / 1000.0
                    
                    newSubtitle.timelineOut = (self?.recorderManager?.voiceModel?.startTime ?? 0.0) + Float64(((payload?["time"]) as? Int) ?? 0) / 1000.0
                    var showText = ((payload?["result"]) as? String) ?? ""
                    if showText.count > subtitleMaxlength {
                        showText = showText.substring(to: subtitleMaxlength)
                        showText += "..."
                    }
                    newSubtitle.text = showText
                    newSubtitle.audioFilePath = audioFilePath ?? ""
                    
                    BFLog(message: "添加字幕数据 timelineIn \(newSubtitle.timelineIn) timelineOut \(newSubtitle.timelineOut) text: \(newSubtitle.text) 音频路径为：\(audioFilePath ?? "")")
                    newSubtitle.setting = self?.subtitleSettingView.subtitle.setting ?? BFSubTitileSettingModel()
                    
                    self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers.append(newSubtitle)
                }
            }
            // 录音结束
            self?.recorderManager?.endRecordHandle = { [weak self] voideModel, _ in
                if let sself = self, let model = voideModel, FileManager.default.fileExists(atPath: model.wavFilePath ?? "") {
                    // 加入到语音数组里
                    model.endTime = sself.currentAssetProgress.seconds
                    
                    let newRange = CMTimeRange(start: CMTime(seconds: model.startTime, preferredTimescale: 1000), end: CMTime(seconds: model.endTime, preferredTimescale: 1000))
                    
                    var deletedVoices = [(PQVoiceModel, Int)]()
                    
                    for (i, m) in sself.itemModels[sself.currItemModelIndex].voiceStickers.enumerated() {
                        let originRange = CMTimeRange(start: CMTime(seconds: m.startTime, preferredTimescale: 1000), end: CMTime(seconds: m.endTime, preferredTimescale: 1000))
                        
                        if CMTimeRangeGetIntersection(originRange, otherRange: newRange).duration.seconds > 0 {
                            deletedVoices.append((m, i))
                            continue
                        }
                    }
                    sself.itemModels[sself.currItemModelIndex].voiceStickers.removeAll { m in
                        let originRange = CMTimeRange(start: CMTime(seconds: m.startTime, preferredTimescale: 1000), end: CMTime(seconds: m.endTime, preferredTimescale: 1000))
                        return CMTimeRangeGetIntersection(originRange, otherRange: newRange).duration.seconds > 0
                    }
                    BFLog(1, message: "添加录音文件:\(model.startTime) -- \(model.endTime)")
                    
                    var event = sself.events.last
                    if event != nil {
                        event!.deletedVoices = deletedVoices
                        sself.events.removeLast()
                        sself.events.append(event!)
                    }
                    sself.itemModels[sself.currItemModelIndex].voiceStickers.append(model)
                    if sself.itemModels[sself.currItemModelIndex].mediaType == .IMAGE {
                        var duration: Double = 0
                        sself.itemModels[sself.currItemModelIndex].voiceStickers.forEach { temp in
                            temp.duration = "\(temp.endTime - temp.startTime)"
                            duration = duration + (temp.endTime - temp.startTime)
                        }
                        sself.itemModels[sself.currItemModelIndex].materialDuraion = Double(String(format: "%.3f", duration)) ?? 0
                        self?.isEndPlay = true
                        // 录制结束显示播放按钮
                        (sself.collectionView.cellForItem(at: IndexPath(item: sself.currItemModelIndex, section: 0)) as? BFImageCoverViewCell)?.playBtn.isSelected = sself.itemModels[sself.currItemModelIndex].voiceStickers.count <= 0
                    }
                    DispatchQueue.main.async { [weak self] in
                        // 录音完，重绘撤销按钮，更新录音按钮，
                        self?.changeWithDrawBtnLayout(true)
                        // 注：在录制结束时矫正当前位置，避免跟指针无法对其
                        self?.indirectionView?.resetCurrentItem(start: model.startTime, end: model.endTime)
                        // 矫正进度
                        self?.resetCurrentProgress()
                        self?.deleteRecordBtn.isHidden = true
                        self?.recordBtn.isHidden = false
                    }
                    sself.currentPlayRecordIndex = -3 // 刚录音完，不需要播放录音
                    // 重置录制开始时间
                    sself.recordStartTime = 0
                }
            }
            self?.recorderManager?.cancelRecordHandle = { [weak self] voiceModel in
                //add by ak 取消录制后删除对应字幕数据，这里可恢复操作吗？
                var subtitleCount = self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers.count ?? 0
                BFLog(2, message: "删除\(voiceModel?.wavFilePath ?? "")对应的字幕 前 count\(subtitleCount)")
                if(subtitleCount > 0){
                    for title in self!.itemModels[self?.currItemModelIndex ?? 0].titleStickers {
                        if title.audioFilePath == voiceModel?.wavFilePath ?? "" {
                            if let index = self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers.firstIndex(of: title) {
                                self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers.remove(at: index)
                            }
                        }
                    }
                }
                subtitleCount = self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers.count ?? 0
                BFLog(2, message: "删除\(voiceModel?.wavFilePath ?? "")对应的字幕  后 count\(subtitleCount)")
                
                // 取消录制以后重置进度
                self?.withdrawAction()
                // 重置录制开始时间
                self?.recordStartTime = 0
                /// 重置进度
                self?.resetCurrentProgress()
                // 移除
                self?.indirectionView?.deleteItem(isCurrent: true)
            }
        }
        
        view.backgroundColor = .black
        view.addSubview(collectionView)
        //        playView = GPUImageView(frame: view.bounds)
        //        view.addSubview(playView!)
        fetchVideo()
        //        view.addSubview(playBtn)
        view.addSubview(bottomeView)
        view.addSubview(avatarView)
        //        view.addSubview(openCameraBtn)
        //        view.addSubview(drawPinBtn)
        view.addSubview(subtitleBtn)
        view.addSubview(soundSettingBtn)
        view.addSubview(subtitleSettingView)
        view.addSubview(subtitleLabel)
        
        view.addSubview(audioSettingView)
        
        bottomeView.addSubview(progreddL)
        //        view.addSubview(toolV)
        bottomeView.addSubview(recordBtn)
        bottomeView.addSubview(deleteRecordBtn)
        bottomeView.addSubview(withDrawBtn)
        //        bottomeView.addSubview(changeVoiceBtn)
        bottomeView.addSubview(progressThumV)
        
   
        
        avatarView.recordEndCallBack = { _, materialsModel in
            BFLog(message: "新录制完成::::\(materialsModel?.locationPath ?? "")")
        }
        
        // 字幕设置回调
        // 设置默认值
        setSubtitleStyle(settingModel: subtitleSettingView.subtitle.setting)
        subtitleSettingView.subtitleSettingCallBack = { [weak self] subtitileModel in
            
            self?.setSubtitleStyle(settingModel: subtitileModel.setting)
        }
        // 编辑字幕完成
        subtitleEditView.editSubtitleDone = { [weak self] newtext, index in
            // 1,刷新 UI
            self?.subtitleLabel.text = newtext
            
            self?.setSubtitleStyle(settingModel: (self?.subtitleSettingView.subtitle.setting)!)
            
            // 更新缓存数据
            if index < (self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers.count ?? 0) {
                if newtext.count == 0 {
                    // 删除数据
                    BFLog(message: "清空字幕操作要删除原字幕 sticker 数据")
                    self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers.remove(at: index)
                } else {
                    // 更新数据
                    self?.itemModels[self?.currItemModelIndex ?? 0].titleStickers[index].text = newtext
                }
            }
        }
        
        layoutsubview()
        
        view.addSubview(subtitleEditView)
        
        if !SWNetRequest.isNetReachabled() {
            cShowHUB(superView: view, msg: "网络不佳，字幕可能无法生成")
        }
    }
    
    @objc func editSubtitle() {
        BFLog(message: "编辑字幕 index:\(showSubtitleIndex)")
        if !isNormalPlaying {
            subtitleEditView.isHidden = false
            subtitleEditView.textView.becomeFirstResponder()
            subtitleEditView.setNewText(text: subtitleLabel.text ?? "", index: showSubtitleIndex)
        } else {
            BFLog(message: "播放状态不可以编辑字幕")
        }
    }
    
    /// 更新字幕，在回放时使用
    /// - Parameter time: 当前播放的进度
    func updateSubtitle(time: CMTime) {
        BFLog(message: "currTime is \(CMTimeGetSeconds(time))")
        var findShowSubtitle: PQEditSubTitleModel?
        for (index, subtitle) in itemModels[currItemModelIndex].titleStickers.enumerated() {
            if subtitle.timelineIn <= CMTimeGetSeconds(time), subtitle.timelineOut >= CMTimeGetSeconds(time) {
                findShowSubtitle = subtitle
                BFLog(message: "找到要显示的字幕 in \(findShowSubtitle?.timelineIn ?? 0.0) out \(findShowSubtitle?.timelineOut ?? 0.0) text:\(findShowSubtitle?.text ?? "") currTime is \(CMTimeGetSeconds(time))")
                showSubtitleIndex = index
                break
            }
        }
        if findShowSubtitle != nil, subtitleSettingView.subtitle.setting.subtitleIsShow {
            subtitleLabel.text = findShowSubtitle?.text ?? ""
            setSubtitleStyle(settingModel: subtitleSettingView.subtitle.setting)
            
        } else {
            subtitleLabel.text = ""
            subtitleLabel.backgroundColor = UIColor.clear
        }
    }
    
    /// 设置字幕样式和位置
    /// - Parameter settingModel: 样式model
    func setSubtitleStyle(settingModel: BFSubTitileSettingModel) {
        if settingModel.subtitleIsShow {
            // 设置样式
            subtitleLabel.strokeColor = settingModel.strokeColor
            subtitleLabel.strokeWidth = settingModel.strokeWidth * 375 / 1080
            subtitleLabel.font = UIFont.boldSystemFont(ofSize: CGFloat(settingModel.subtitleSize) * 375 / 1080)
            subtitleLabel.textColor = settingModel.fontColor
            
            let leftPoint: CGFloat = 37
            
            let height = sizeTextFits(attributedText: nil, text: subtitleLabel.text ?? "", numberOfLines: 0, font: subtitleLabel.font, maxSize: CGSize(width: cScreenWidth - leftPoint * 2, height: CGFloat.greatestFiniteMagnitude)).height
            
            subtitleLabel.backgroundColor = subtitleSettingView.subtitle.setting.backgroundColor
            // 下 //设置位置
            if subtitleSettingView.subtitle.setting.subtitlePoint == 0 {
                subtitleLabel.frame = CGRect(x: leftPoint, y: cScreenHeigth * 0.70 - height, width: cScreenWidth - 37 * 2, height: height)
            } else if subtitleSettingView.subtitle.setting.subtitlePoint == 1 { // 中
                subtitleLabel.frame = CGRect(x: leftPoint, y: cScreenHeigth / 2.0 - height / 2.0, width: cScreenWidth - 37 * 2, height: height)
            } else { // 上
                subtitleLabel.frame = CGRect(x: leftPoint, y: cScreenHeigth * 0.12, width: cScreenWidth - 37 * 2, height: height)
            }
        }
    }
    
    func layoutsubview() {
        bottomeView.snp.makeConstraints { make in
            make.left.bottom.right.equalToSuperview()
            make.height.equalTo(adapterWidth(width: 220))
        }
        
        progreddL.snp.makeConstraints { make in
            make.width.equalTo(100)
            make.centerX.equalToSuperview()
            make.bottom.equalTo(progressThumV.snp.top)
            make.height.equalTo(18)
        }
        
        withDrawBtn.snp.makeConstraints { make in
            make.left.equalToSuperview()
            make.width.height.equalTo(65)
            make.top.equalTo(128)
        }
        //        changeVoiceBtn.snp.makeConstraints { make in
        //            make.right.equalToSuperview()
        //            make.top.width.height.equalTo(withDrawBtn)
        //        }
        
        recordBtn.snp.makeConstraints { make in
            make.left.equalTo(withDrawBtn.snp.right)
            make.right.equalTo(-65)
            make.height.equalTo(42)
            make.top.equalTo(withDrawBtn).offset(6)
        }
        deleteRecordBtn.snp.makeConstraints { make in
            make.left.equalTo(withDrawBtn.snp.right)
            make.right.equalTo(-65)
            make.height.equalTo(42)
            make.top.equalTo(withDrawBtn).offset(6)
        }
        
        //        openCameraBtn.snp.makeConstraints { make in
        //            make.right.equalToSuperview().offset(-12)
        //            make.top.equalToSuperview().offset(98)
        //            make.width.equalTo(80)
        //            make.height.equalTo(124)
        //        }
        //
        //        drawPinBtn.snp.makeConstraints { make in
        //            make.right.equalTo(openCameraBtn)
        //            make.top.equalTo(openCameraBtn.snp.bottom).offset(18)
        //            make.width.equalTo(80)
        //            make.height.equalTo(124)
        //        }
        
        subtitleBtn.snp.makeConstraints { make in
            make.right.equalToSuperview().offset(-12)
            make.top.equalToSuperview().offset(98)
            make.width.equalTo(40)
            make.height.equalTo(62)
        }
        
        soundSettingBtn.snp.makeConstraints { make in
            make.right.equalTo(subtitleBtn)
            make.top.equalTo(subtitleBtn.snp.bottom).offset(18)
            make.width.equalTo(40)
            make.height.equalTo(62)
        }
        
        withDrawBtn.imageEdgeInsets = UIEdgeInsets(top: -withDrawBtn.imageView!.height, left: 0, bottom: 0, right: -withDrawBtn.titleLabel!.width)
        withDrawBtn.titleEdgeInsets = UIEdgeInsets(top: withDrawBtn.titleLabel!.height + 2, left: -withDrawBtn.imageView!.width, bottom: 0, right: 0)
        
        changeVoiceBtn.imageEdgeInsets = UIEdgeInsets(top: -changeVoiceBtn.imageView!.height - 2, left: 0, bottom: 0, right: -changeVoiceBtn.titleLabel!.width)
        changeVoiceBtn.titleEdgeInsets = UIEdgeInsets(top: changeVoiceBtn.titleLabel!.height + 2, left: -changeVoiceBtn.imageView!.width, bottom: 0, right: 0)
    }
    
    //    public override func viewWillLayoutSubviews() {
    //        super.viewWillLayoutSubviews()
    //
    //    }
    
    // MARK: - 按钮事件响应
    
    override public func backBtnClick() {
        pause()
    }
    
    // 触发拖曳手势后，执行的动作
    @objc func pan(recognizer: UIPanGestureRecognizer) {
        // 设置 UIView 新的位置
        if !checkStatus(show: false) {
            return
        }
        
        let point = recognizer.location(in: bottomeView)
        
        switch recognizer.state {
        case .began:
            touchStart = recognizer.location(in: bottomeView)
            beginOnStartBtn = recordBtn.frame.contains(touchStart)
        case .changed:
            if beginOnStartBtn == true {
                let nowPoint = recognizer.location(in: bottomeView)
                BFLog(1, message: "nowPoint x: \(nowPoint.x) y:\(nowPoint.y)")
                if recordBtn.frame.contains(nowPoint) {
                } else {}
            }
            
        case .ended:
            BFLog(1, message: "移动结束 \(beginOnStartBtn)")
            if beginOnStartBtn == true {
                beginOnStartBtn = false
                endRecord()
                //                if recordBtn.frame.contains(point) {
                //                    // 结束录制
                //                } else {
                //                    cancleRecord()
                //                }
            }
            
        default:
            break
        }
    }
    
    @objc func openCamera() {
        if avatarView.isHidden {
            avatarView.isHidden = false
            avatarView.openCamera()
        } else {
            avatarView.isHidden = true
            avatarView.closeCamera()
        }
    }
    
    @objc func drawPin() {}
    
    // 字幕
    @objc func subTitleClick() {
        BFLog(message: "subTitle Click ")
        subtitleSettingView.isHidden = !subtitleSettingView.isHidden
        if subTitleBtnClickHandle != nil {
            subTitleBtnClickHandle!(subtitleSettingView.subtitle.setting.subtitleIsShow)
        }
    }
    
    // 声音设置
    @objc func soundSetting() {
        BFLog(message: "设置声音")
        
        audioSettingView.isHidden = false
    }
    
    /// 删除指定段落的所有字幕 数据
    /// - Parameter voiceModel: 删除的音频数据
    func deleteTitles(voiceModel: PQVoiceModel) {
        BFLog(message: "itemModels[currItemModelIndex].titleStickers  删除前：\(itemModels[currItemModelIndex].titleStickers.count) model.startTime: \(voiceModel.startTime) model.end: \(voiceModel.endTime)")
        for title in itemModels[currItemModelIndex].titleStickers {
            if let index = itemModels[currItemModelIndex].titleStickers.firstIndex(of: title) {
                if title.timelineIn >= voiceModel.startTime, title.timelineOut <= voiceModel.endTime {
                    // 删除前数据存入缓存
                    let sutbitle = itemModels[currItemModelIndex].titleStickers[index]
                    events.append(WithDrawModel(type: 4, timestamp: currentAssetProgress.seconds, deletedTittles: [(sutbitle, isStopAtRecordRange)]))
                    
                    itemModels[currItemModelIndex].titleStickers.remove(at: index)
                }
            }
        }
        BFLog(message: "itemModels[currItemModelIndex].titleStickers  删除后：\(itemModels[currItemModelIndex].titleStickers.count)")
        
        // 清空字幕UI
        subtitleLabel.text = ""
        subtitleLabel.backgroundColor = UIColor.clear
    }
    
    @objc func deleteRecorded() {
        if !isDragingProgressSlder, isStopAtRecordRange != -1, isStopAtRecordRange < itemModels[currItemModelIndex].voiceStickers.count {
            let model = itemModels[currItemModelIndex].voiceStickers[isStopAtRecordRange]
            itemModels[currItemModelIndex].voiceStickers.remove(at: isStopAtRecordRange)
            events.append(WithDrawModel(type: 3, timestamp: currentAssetProgress.seconds, deletedVoices: [(model, isStopAtRecordRange)]))
            indirectionView?.deleteItem(index: isStopAtRecordRange)
            // 注：删除录音后图片素材需要回撤指针进度,同时后面录音往前挪
            if itemModels[currItemModelIndex].mediaType == .IMAGE {
                let currDuration = model.endTime - model.startTime
                itemModels[currItemModelIndex].materialDuraion = itemModels[currItemModelIndex].materialDuraion - currDuration
                currentAssetProgress = CMTime(seconds: model.startTime, preferredTimescale: 1000)
                // 更新进度
                resetCurrentProgress()
                for (index, item) in itemModels[currItemModelIndex].voiceStickers.enumerated() {
                    if index >= isStopAtRecordRange {
                        // 注：开始时间减去duration or 等一前一段录音的结束时间
                        //                        item.startTime = item.startTime - currDuration
//                        item.startTime = itemModels[currItemModelIndex].voiceStickers[index - 1].endTime
//                        item.endTime = item.startTime + (Double(item.duration ?? "0") ?? 0)
                        item.startTime -= currDuration
                        item.endTime -= currDuration
                    }
                }
                // 重绘录音进度视图
                indirectionView?.resetAllSubViews(items: itemModels[currItemModelIndex].voiceStickers, percenWidth: progressThumV.thumbImageWidth / 2.0, totalDuration: itemModels[currItemModelIndex].materialDuraion)
            }
            searchStopAtRecordRange()
            deleteTitles(voiceModel: model)
        }
    }
    
    @objc func startRecord() {
        //        BFLog(1, message: "start \(UIControl.Event.touchDown)")
        if(!checkStatus(show:true)){
            BFLog(message: "没有录音权限不能录制")
            return
        }
        
        // 停止进度条滚动
        if isDragingProgressSlder {
            return
        }
        self.progressThumV.progressView.isUserInteractionEnabled = false
        self.collectionView.isScrollEnabled = false
        pause()
        isRecording = true
        
        if recorderManager == nil {
            BFLog(message: "录音机初始化错误！！！")
            return
        }
        
        let model = PQVoiceModel()
        // 开始时间
        model.startTime = currentAssetProgress.seconds
        model.volume = 100
        recorderManager?.voiceModel = model
        recorderManager?.startRecord()
        if recordStartTime <= 0 {
            recordStartTime = currentAssetProgress.seconds
        }
        // 添加撤销记录点
        events.append(WithDrawModel(type: 2, timestamp: model.startTime))
        
        isRecording = true
        
        if !avatarView.isHidden {
            avatarView.beginRecord()
        }
        if itemModels[currItemModelIndex].mediaType == .VIDEO {
            if !movieIsProcessing {
                movie?.startProcessing()
                movieIsProcessing = true
            }
            assetPlayer?.volume = 0
            assetPlayer?.play()
        }
        // 录制中不显示播放按钮
        (collectionView.cellForItem(at: IndexPath(item: currItemModelIndex, section: 0)) as? BFImageCoverViewCell)?.playBtn.isSelected = true
    }
    
    @objc func endRecord() {
        // 存储录音
        isRecording = false
        pause()
        self.progressThumV.progressView.isUserInteractionEnabled = true
        self.collectionView.isScrollEnabled = true

        recorderManager?.stopRecord(isCancel: false)
        
        if !avatarView.isHidden {
            avatarView.endRecord()
        }
    }
    
    @objc func cancleRecord() {
        isRecording = false
        recorderManager?.stopRecord(isCancel: true)
        self.progressThumV.progressView.isUserInteractionEnabled = true
        self.collectionView.isScrollEnabled = true

        pause()
    }
    
    // 撤销
    @objc func withdrawAction() {
        pause()
        if let action = events.last {
            var jumpTime = action.timestamp
            if action.type == 2 {
                // 撤销录制
                if let modelIndex = itemModels[currItemModelIndex].voiceStickers.firstIndex(where: { mod in
                    mod.startTime == action.timestamp
                }) {
                    let model = itemModels[currItemModelIndex].voiceStickers[modelIndex]
                    itemModels[currItemModelIndex].voiceStickers.remove(at: modelIndex)
                    // 移除
                    indirectionView?.deleteItem(index: modelIndex)
                    var tuples = action.deletedVoices
                    if tuples != nil, tuples!.count > 0 {
                        tuples!.sort { tuple1, tuple2 in
                            tuple1.1 < tuple2.1
                        }
                        tuples?.forEach { tuple in
                            itemModels[currItemModelIndex].voiceStickers.insert(tuple.0, at: tuple.1)
                        }
                    }
                    // 恢复字幕
                    let titleTuples = action.deletedTittles
                    if titleTuples != nil, titleTuples!.count > 0 {
                        titleTuples?.forEach { titleTuple in
                            itemModels[currItemModelIndex].titleStickers.insert(titleTuple.0, at: titleTuple.1)
                        }
                    }
                    jumpTime = model.startTime
                    
                    if itemModels[currItemModelIndex].mediaType == .IMAGE {
                        itemModels[currItemModelIndex].materialDuraion = jumpTime
                    }
                }
            } else if action.type == 3 {
                // 删除录音
                var tuples = action.deletedVoices
                if tuples != nil, tuples!.count > 0 {
                    tuples!.sort { tuple1, tuple2 in
                        tuple1.1 < tuple2.1
                    }
                    tuples?.forEach { tuple in
                        itemModels[currItemModelIndex].voiceStickers.insert(tuple.0, at: tuple.1)
                    }
                }
                
                // 恢复字幕
                let titleTuples = action.deletedTittles
                if titleTuples != nil, titleTuples!.count > 0 {
                    titleTuples?.forEach { titleTuple in
                        itemModels[currItemModelIndex].titleStickers.insert(titleTuple.0, at: titleTuple.1)
                    }
                }
                if itemModels[currItemModelIndex].mediaType == .IMAGE {
                    itemModels[currItemModelIndex].materialDuraion = itemModels[currItemModelIndex].voiceStickers.last?.endTime ?? 0
                }
            } else {}
            events.removeLast()
            
            let dur = itemModels[currItemModelIndex].materialDuraion
            if dur > 0 {
                changeProgress(progress: Float(jumpTime / dur))
            }else {
                changeProgress(progress: Float(0))
            }
            isDragingProgressSlder = false
            currentPlayRecordIndex = -1
            hadPrepareToPlayRecord = false
            progressThumV.progress = jumpTime
            
            if let event = events.last, event.type == 2 {
                changeWithDrawBtnLayout(true)
            } else {
                changeWithDrawBtnLayout(false)
            }
            searchStopAtRecordRange()
        }
    }
    
    @objc func changeVoiceAction() {
        //        nextActionHandle?()
        pause()
    }
    
    @objc func playVideo(btn: UIButton) {
        if !btn.isSelected {
            pause()
            searchStopAtRecordRange()
        } else {
            events.append(WithDrawModel(type: 1, timestamp: currentAssetProgress.seconds))
            changeWithDrawBtnLayout(false)
            play()
        }
    }
    
    @objc func sliderTouchBegan(sender _: UISlider) {
        isDragingProgressSlder = true
        pause()
    }
    
    @objc func sliderTouchEnded(sender: UISlider) {
        changeProgress(progress: sender.value)
        isDragingProgressSlder = false
        currentPlayRecordIndex = -1
        hadPrepareToPlayRecord = false
    }
    
    @objc func sliderValueDidChanged(sender: UISlider) {
        changeProgress(progress: sender.value)
    }
    
    func thumbViewEnded(progress: Float) {
        changeProgress(progress: progress)
        isDragingProgressSlder = false
        currentPlayRecordIndex = -1
        hadPrepareToPlayRecord = false
        BFLog(1, message: "isDragingProgressSlder : \(isDragingProgressSlder)")
        searchStopAtRecordRange(needAdsorb: true)
        changeWithDrawBtnLayout(false)
    }
    
    // 是否吸附在录音首尾处
    func searchStopAtRecordRange(needAdsorb: Bool = false) {
        pauseTime = currentAssetProgress.seconds
        
        // TODO: 滑动，播放暂停，撤销时，判断是否停止录音区间，是则删除相关录音，画笔，头像，字幕
        let elems = itemModels[currItemModelIndex].voiceStickers.enumerated().filter { elem in
            elem.1.startTime <= self.currentAssetProgress.seconds && elem.1.endTime > self.currentAssetProgress.seconds
        }
        
        isEndPlay = false
        
        if elems.count > 0 {
            //  TODO: 停在了录音区间,显示删除按钮
            if needAdsorb {
                if fabs(elems[0].1.endTime - currentAssetProgress.seconds) < 0.5 {
                    BFLog(1, message: "吸附在录音结尾")
                    //                changeWithDrawBtnLayout(false)
                    changeProgress(progress: Float(elems[0].1.endTime / itemModels[currItemModelIndex].materialDuraion))
                    progressThumV.progress = elems[0].1.endTime
                    
                    deleteRecordBtn.isHidden = true
                    recordBtn.isHidden = false
                    isStopAtRecordRange = -1
                    BFLog(1, message: "停在了录音区间外 \(isStopAtRecordRange)")
                } else {
                    if fabs(elems[0].1.startTime - currentAssetProgress.seconds) < 0.5 {
                        BFLog(1, message: "吸附在录音开始")
                        //                    changeWithDrawBtnLayout(true)
                        changeProgress(progress: Float(elems[0].1.startTime / itemModels[currItemModelIndex].materialDuraion))
                        progressThumV.progress = elems[0].1.startTime
                    }
                    deleteRecordBtn.isHidden = false
                    recordBtn.isHidden = true
                    isStopAtRecordRange = elems.first!.0
                    
                    BFLog(1, message: "停在了录音区间里 \(isStopAtRecordRange)")
                }
            } else {
                deleteRecordBtn.isHidden = false
                recordBtn.isHidden = true
                
                isStopAtRecordRange = elems.first!.0
                BFLog(1, message: "停在了录音区间里 \(isStopAtRecordRange)")
            }
            
        } else {
            deleteRecordBtn.isHidden = true
            recordBtn.isHidden = false
            
            isStopAtRecordRange = -1
            BFLog(1, message: "停在了录音区间外 \(isStopAtRecordRange)")
        }
    }
    
    // MARK: - 权限申请
    
    func checkStatus(show: Bool = true) -> Bool {
        let status = AVCaptureDevice.authorizationStatus(for: .audio)
        switch status {
        case .denied, .restricted:
            
            BFLog(message: "麦克风权限被拒绝，请在设置中打开")
            if show {
                let remindData = BFBaseModel()
                remindData.title = "需要开启「麦克风权限」才可以进行录制"
                remindData.summary = ""
                let remindView = BFRemindView(frame: CGRect(x: 0, y: 0, width: cScreenWidth, height: cScreenHeigth))
                remindView.isBanned = true
                remindView.confirmBtn.setTitle("去开启", for: .normal)
                remindView.confirmBtn.setTitleColor(UIColor.hexColor(hexadecimal: "#389AFF"), for: .normal)
                UIApplication.shared.keyWindow?.addSubview(remindView)
                remindView.remindData = remindData
                remindView.cancelBtn.setTitleColor(UIColor.hexColor(hexadecimal: "#389AFF"), for: .normal)
                remindView.remindBlock = { item, _ in
                    if item.tag == 2 {
                        openAppSetting()
                    }
                }
            }
            return false
        case .authorized:
            return true
        case .notDetermined:
            if show {
                requestMicroPhoneAuth()
            }
            return false
        default:
            break
        }
        
        return false
    }
    
    func requestMicroPhoneAuth() {
        AVCaptureDevice.requestAccess(for: .audio) { granted in
            if !granted {
                BFLog(message: "麦克风权限被拒绝，请在设置中打开")
            }
        }
    }
    
    // MARK: - 音视频处理
    
    func playRecord(at currentT: CMTime, periodicTimeObserver: @escaping (_ time: CMTime, _ currentItem: AVPlayerItem) -> Void, didPlayToEndTime: @escaping (_ recordedInfo: (Int, PQVoiceModel)?, _ currentItem: AVPlayerItem?) -> Void, playFailed: @escaping (_ recordedInfo: (Int, PQVoiceModel)?, _ currentItem: AVPlayerItem?) -> Void) {
        if currentPlayRecordIndex == -3 { // 刚录音完，不需要播放
            return
        }
        let list = itemModels[currItemModelIndex].voiceStickers.sorted { m1, m2 in
            m1.startTime < m2.startTime
        }
        let (shouldPlayRecordIndex, recordedAudio) = list.enumerated().first { model in
            model.1.endTime > CMTimeGetSeconds(currentT)
        } ?? (-1, nil)
        
        guard let recordedAudio = recordedAudio, recordedAudio.wavFilePath.count > 0 else {
            return
        }
        BFLog(1, message: "当前时间：\(CMTimeGetSeconds(currentT)), 找到的音频：\(recordedAudio.startTime), \(recordedAudio.endTime), \(recordedAudio.wavFilePath ?? "")")
        
        // 创建播放器
        if recordPlayer == nil || (recordPlayer?.currentItem?.asset as? AVURLAsset)?.url.lastPathComponent != (recordedAudio.wavFilePath as NSString?)?.lastPathComponent {
            let newItem = AVPlayerItem(url: URL(fileURLWithPath: recordedAudio.wavFilePath))
            BFLog(1, message: "录音播放器初始化：\(recordPlayer == nil ? "init player" : "replace item")")
            
            if let player = recordPlayer {
                player.pause()
                if let playItem = player.currentItem {
                    NotificationCenter.default.removeObserver(self, name: .AVPlayerItemDidPlayToEndTime, object: playItem)
                    recordPlayer?.replaceCurrentItem(with: newItem)
                }
            } else {
                recordPlayer = AVPlayer(playerItem: newItem)
            }
            recordPlayer!.volume = 1
            //                self.recordPlayer?.prepareToPlay()
            currentPlayRecordIndex = -1
            hadPrepareToPlayRecord = false
            BFLog(1, message: "录音播放器初始化(有时候不准)")
            
            NotificationCenter.default.addObserver(forName: .AVPlayerItemDidPlayToEndTime, object: newItem, queue: .main) { [weak self, weak recordedAudio] _ in
                self?.hadPrepareToPlayRecord = false
                self?.currentPlayRecordIndex = -1
                self?.assetPlayer?.volume = self?.noSpeakVolume ?? 0
                //                self?.deleteRecordBtn.isHidden = true
                //                self?.recordBtn.isHidden = false
                didPlayToEndTime((shouldPlayRecordIndex, recordedAudio) as? (Int, PQVoiceModel), newItem)
            }
            avplayerTimeObserver?.invalidate()
            avplayerTimeObserver = recordPlayer?.addPeriodicTimeObserver(forInterval: CMTime(value: 1, timescale: 1000), queue: DispatchQueue.global()) { [weak self] time in
                BFLog(3, message: "当前播放---\(time),\(time.seconds)")
                if CMTimeGetSeconds(self?.currenStartPlayTime ?? CMTime.zero) <= 0 {
                    self?.currenStartPlayTime = time
                }
                periodicTimeObserver(CMTime(seconds: CMTimeGetSeconds(time) - CMTimeGetSeconds(self?.currenStartPlayTime ?? CMTime.zero), preferredTimescale: 1000), newItem)
            } as? NSKeyValueObservation
        }
        if recordPlayer?.currentItem?.duration.timescale == 0 {
            BFLog(1, message: "时间timescale  == 0")
            playFailed((shouldPlayRecordIndex, recordedAudio) as? (Int, PQVoiceModel), recordPlayer?.currentItem)
            return
        }
        synced(currentPlayRecordIndex) { [weak self] in
            guard let sself = self else {
                return
            }
            BFLog(1, message: "判断是否开始录音播放** hadPrepareToPlayRecord:\(hadPrepareToPlayRecord), currentPlayRecordIndex:\(currentPlayRecordIndex), isNormalPlaying :\(sself.isNormalPlaying),\(recordPlayer?.currentItem?.duration.timescale ?? 0),\(CMTimeGetSeconds(currentT) >= recordedAudio.startTime),\(CMTimeGetSeconds(currentT) <= recordedAudio.endTime - 0.2)")
            
            if !hadPrepareToPlayRecord,
               recordPlayer?.currentItem?.duration.timescale != 0,
               CMTimeGetSeconds(currentT) >= (recordedAudio.startTime - 0.1),
               CMTimeGetSeconds(currentT) <= recordedAudio.endTime - 0.2 // 这个条件是避免录音结束后有小幅度回退导致播放最新录音
            {
                // 应当开始播放了
                // 两个逻辑：如果在播，则跳过;如果暂停拖动到中间，则seek
                if currentPlayRecordIndex == -1, sself.isNormalPlaying {
                    let second = CMTimeGetSeconds(currentT) - recordedAudio.startTime
                    sself.assetPlayer?.volume = sself.haveSpeakVolume
                    
                    DispatchQueue.main.async { [weak sself] in
                        self?.recordPlayer?.seek(to: CMTime(value: CMTimeValue(second * 1_000_000), timescale: 1_000_000), toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000), completionHandler: { [weak self] finished in
                            if finished, self?.isNormalPlaying ?? false {
                                self?.recordPlayer?.play()
                                //                                self?.deleteRecordBtn.isHidden = false
                                //                                self?.recordBtn.isHidden = true
                                BFLog(1, message: "录音开始播放 playing, \(second), \(CMTimeGetSeconds(self?.recordPlayer?.currentItem?.duration ?? .zero)), \(self?.recordPlayer?.currentItem?.currentTime().seconds ?? 0)")
                            }
                        })
                    }
                    currentPlayRecordIndex = shouldPlayRecordIndex
                    hadPrepareToPlayRecord = true
                    BFLog(1, message: "录音开始播放2, \(second), \(CMTimeGetSeconds(recordPlayer?.currentItem?.duration ?? .zero))")
                }
            }
        }
        BFLog(1, message: "应当播放：\(shouldPlayRecordIndex), 当前播放：\(currentPlayRecordIndex)")
    }
    
    func play() {
        BFLog(1, message: "开始播放 \(currentAssetProgress.seconds)")

        isNormalPlaying = true
        if isEndPlay {
            isEndPlay = false
            assetPlayer?.seek(to: CMTime.zero)
            progressThumV.progress = 0
            currentPlayRecordIndex = -1
            if itemModels[currItemModelIndex].mediaType == .VIDEO {
                recordBtn.isHidden = false
            }
            currentAssetProgress = CMTime.zero
        }
        if itemModels[currItemModelIndex].mediaType == .VIDEO {
            assetPlayer?.volume = noSpeakVolume
            if !movieIsProcessing {
                movie?.startProcessing()
                movieIsProcessing = true
            }
            assetPlayer?.play()
        } else {
            // 处理图片音频播放
            imageRecordPlay()
        }
        
        deleteRecordBtn.isHidden = true
        
        subtitleBtn.isHidden = true
        soundSettingBtn.isHidden = true
        withDrawBtn.isHidden = true
        recordBtn.isHidden = true
    }
    
    func pause() {
        BFLog(1, message: "暂停播放")
        isNormalPlaying = false
        
        subtitleBtn.isHidden = false
        soundSettingBtn.isHidden = false
        withDrawBtn.isHidden = false
        recordBtn.isHidden = false
        //        movie?.cancelProcessing()
        assetPlayer?.pause()
        recordPlayer?.pause()
        recordStartPlayTime = CMTime.zero
        pauseTime = currentAssetProgress.seconds
        
        assetPlayer?.seek(to: currentAssetProgress, toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000), completionHandler: { _ in
        })
        // 暂停状态
        (collectionView.cellForItem(at: IndexPath(item: currItemModelIndex, section: 0)) as? BFImageCoverViewCell)?.playBtn.isSelected = (itemModels[currItemModelIndex].mediaType == .IMAGE && itemModels[currItemModelIndex].voiceStickers.count <= 0)
    }
    
    func fetchVideo() {
        if assets.count > 0 {
            currItemModelIndex = 0
            for (index, asset) in assets.enumerated() {
                let itemModel = BFRecordItemModel()
                itemModel.index = index
                itemModel.initOriginData(phasset: asset)
                itemModels.append(itemModel)
                if index == 0 {
                    if asset.mediaType == .video {
                        itemModel.fetchAVUrlAssetCallBack = { [weak self, weak itemModel] _ in
                            //                            self?.export(avsss:uralss)
                            DispatchQueue.main.async { [weak self, weak itemModel] in
                                self?.progressThumV.recordItem = itemModel
                                self?.progressThumV.isHidden = false
                            }
                        }
                    }
                }
            }
            collectionView.reloadData()
        }
    }
    
    func setCoverImage(img: UIImage) {
        if let pic = GPUImagePicture(image: img) {
            let filter = GPUImageFilter()
            pic.addTarget(filter)
            filter.addTarget(playView)
            pic.processImage()
        }
    }
    
    func setVideoPlay(item: AVPlayerItem?, imageView: GPUImageView?) {
        guard let playerItem = item else {
            return
        }
        guard let preView = imageView else {
            return
        }
        if movie != nil {
            cleanMovieTarget()
        }
        movie = GPUImageMovie(playerItem: playerItem)
        //        movie?.runBenchmark = true
        movie?.playAtActualSpeed = true
        
        let filter = GPUImageFilter()
        movie?.addTarget(filter)
        filter.addTarget(preView)
        
        movie?.startProcessing()
        movieIsProcessing = true
    }
    
    func setAudioPlay(item: AVPlayerItem?) {
        guard let item = item else {
            return
        }
        if let playItem = assetPlayer?.currentItem {
            NotificationCenter.default.removeObserver(self, name: .AVPlayerItemDidPlayToEndTime, object: playItem)
            assetPlayer?.replaceCurrentItem(with: item)
        } else {
            assetPlayer = AVPlayer(playerItem: item)
            assetPlayer?.volume = noSpeakVolume
            
            avplayerTimeObserver?.invalidate()
            avplayerTimeObserver = assetPlayer?.addPeriodicTimeObserver(forInterval: CMTime(value: 1, timescale: 1000), queue: DispatchQueue.global()) { [weak self] time in
                // 进度监控
                self?.periodicTimeObserver(item: item, time: time)
                
                if self?.isNormalPlaying ?? false {
                    // 播放对应的录音音频
                    self?.playRecord(at: time, periodicTimeObserver: { currentT, currentItem in
                        BFLog(message: "播放一段进度：\(currentT),\(currentItem)")
                    }, didPlayToEndTime: { recordInfo, _ in
                        BFLog(message: "播放一段结束：\(recordInfo?.1.endTime ?? 0)")
                    }, playFailed: { _, _ in })
                }
            } as? NSKeyValueObservation
        }
        
        NotificationCenter.default.addObserver(forName: .AVPlayerItemDidPlayToEndTime, object: assetPlayer?.currentItem, queue: .main) { [weak self] notify in
            BFLog(1, message: "AVPlayerItemDidPlayToEndTime = \(notify)")
            if (self?.currItemModelIndex ?? 0) < ((self?.itemModels.count ?? 0) - 1) {
                self?.collectionView.setContentOffset(CGPoint(x: CGFloat((self?.currItemModelIndex ?? 0) + 1) * (self?.collectionView.frame.width ?? 0), y: 0), animated: true)
            } else {
                self?.isNormalPlaying = false
                //            self?.assetPlayer?.seek(to: CMTime.zero)
                //            self?.progressThumV.progress = 0
                self?.isEndPlay = true
                //            self?.currentPlayRecordIndex = -1
                self?.recordBtn.isHidden = true
            }
            if self?.isRecording ?? false {
                self?.endRecord()
                cShowHUB(superView: nil, msg: "此视频已录制到头了哦!")
            }
            (self?.collectionView.cellForItem(at: IndexPath(item: self!.currItemModelIndex, section: 0)) as? BFImageCoverViewCell)?.playBtn.isSelected = false
        }
    }
    
    /// 音视频播放进度-视频跟图片单独处理
    /// - Parameter CMTime: <#CMTime description#>
    func periodicTimeObserver(item: AVPlayerItem?, time: CMTime) {
        // 进度监控
        //        if time.seconds < pauseTime {
        //            return
        //        }
        if itemModels[currItemModelIndex].mediaType == .VIDEO {
            if CMTimeGetSeconds(item?.duration ?? CMTime.zero) > 0 {
                currentAssetProgress = CMTime(seconds: time.seconds, preferredTimescale: 1000)
                BFLog(1, message: "curr:\(CMTimeGetSeconds(currentAssetProgress))")
                DispatchQueue.main.async { [weak self] in
                    self?.progreddL.text = String(format: "%@", CMTimeGetSeconds(time).formatDurationToHMS())
                    let su = !(self?.isDragingProgressSlder ?? false) || (self?.isRecording ?? false) || (self?.isNormalPlaying ?? false)
                    if su {
                        self?.progressThumV.progress = time.seconds
                    }
                    // 更新字幕
                    self?.updateSubtitle(time: time)
                }
            }
        }
    }
    
    func cleanMovieTarget() {
        movie?.cancelProcessing()
        movieIsProcessing = false
        movie?.targets().forEach { target in
            if let objc = target as? GPUImageOutput {
                objc.removeAllTargets()
            }
        }
        movie?.removeAllTargets()
        movie?.removeFramebuffer()
        GPUImageContext.sharedFramebufferCache().purgeAllUnassignedFramebuffers()
    }
    
    // MARK: - 录音对应图像绘制
    
    // 撤销按钮修改title，重绘
    func changeWithDrawBtnLayout(_ recorded: Bool) {
        if recorded {
            withDrawBtn.setTitle("撤销录制", for: .normal)
        } else {
            withDrawBtn.setTitle("回退", for: .normal)
        }
        withDrawBtn.imageEdgeInsets = UIEdgeInsets(top: -withDrawBtn.imageView!.height, left: 0, bottom: 0, right: -withDrawBtn.titleLabel!.width)
        withDrawBtn.titleEdgeInsets = UIEdgeInsets(top: withDrawBtn.titleLabel!.height + 2, left: -withDrawBtn.imageView!.width, bottom: 0, right: 0)
        
        withDrawBtn.setNeedsLayout()
        withDrawBtn.layoutIfNeeded()
    }
    
    // 通过缩略图进度条控制播放进度
    func changeProgress(progress: Float) {
        if itemModels[currItemModelIndex].mediaType == .VIDEO {
            if let duration = assetPlayer?.currentItem?.duration {
                currentAssetProgress = CMTime(value: CMTimeValue(progress * Float(CMTimeGetSeconds(duration)) * 1000), timescale: 1000)
                DispatchQueue.main.async { [weak self] in
                    self!.progreddL.text = String(format: "%@", CMTimeGetSeconds(self!.currentAssetProgress).formatDurationToHMS())
                }
                assetPlayer!.seek(to: currentAssetProgress, toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000)) { _ in
                }
            }
        } else {
            if itemModels[currItemModelIndex].materialDuraion > 0 {
                currentAssetProgress = CMTime(value: CMTimeValue(progress * Float(itemModels[currItemModelIndex].materialDuraion) * 1000), timescale: 1000)
                DispatchQueue.main.async { [weak self] in
                    self?.progreddL.text = String(format: "%@", CMTimeGetSeconds(self!.currentAssetProgress).formatDurationToHMS())
                }
                BFLog(message: "progress = \(progress),currentAssetProgress = \(currentAssetProgress.seconds),materialDuraion = \(itemModels[currItemModelIndex].materialDuraion)")
            }
        }
    }
    
    func drawProgressIndication(progress: Double) {
        if indirectionView == nil {
            indirectionView = BFIndirectionProgressView(frame: progressThumV.progessIndicateBackV.bounds, percenWidth: itemModels[currItemModelIndex].mediaType == .IMAGE ? progressThumV.thumbImageWidth / 2 : 0, totalDuration: itemModels[currItemModelIndex].materialDuraion)
            progressThumV.progessIndicateBackV.addSubview((indirectionView)!)
        }
        // 更新录制进度
        // 注：视频无法以录制进度驱动，因当录音开始录音时播放器还未播放，导致进度不一致
        // 注：在录制停止时，视频播放器进度依然在走，误差在80毫秒左右
        if isRecording {
            let ratioX = 0.08
            indirectionView?.setProgress(start: recordStartTime, progress: progress - ratioX)
        }
        if itemModels[currItemModelIndex].mediaType == .IMAGE {
            imageRecordProgress(isRecord: true, progress: progress)
        }
    }
    
    // 修正视频旋转方向，因为自己录制的竖屏视频会预览为横屏
    func reloadMaterial() {
        let recordItem = itemModels[currItemModelIndex]
        if  let vasset = recordItem.videoAsset, let cell: BFImageCoverViewCell = collectionView.cellForItem(at: IndexPath(item: currItemModelIndex, section: 0)) as? BFImageCoverViewCell {
            setVideoPlay(item: recordItem.playItem, imageView: cell.playView)
            setAudioPlay(item: recordItem.playItem)
            playBtn = cell.playBtn
//            recordItem.videoAsset
            
            let degress = degressFromVideoFile(asset: vasset)
            switch degress {
            case 90:
                cell.playView.setInputRotation(GPUImageRotationMode(rawValue: 2), at: 0)
            case 180:
                cell.playView.setInputRotation(GPUImageRotationMode(rawValue: 7), at: 0)
            case 270:
                cell.playView.setInputRotation(GPUImageRotationMode(rawValue: 1), at: 0)
            default:
                break
            }
        }
    }
}

extension BFRecordScreenController: GPUImageMovieDelegate {
    public func didCompletePlayingMovie() {
        BFLog(1, message: "播放结束")
        currentPlayRecordIndex = -1
    }
}

extension BFRecordScreenController: AVAudioRecorderDelegate {
    public func audioRecorderDidFinishRecording(_: AVAudioRecorder, successfully _: Bool) {
        BFLog(1, message: "录音结束")
    }
}

extension BFRecordScreenController: AVAudioPlayerDelegate {
    public func audioPlayerDidFinishPlaying(_: AVAudioPlayer, successfully _: Bool) {
        BFLog(1, message: "录音播放结束")
    }
}

// MARK: - UICollectionViewDelegate

/// UICollectionViewDelegate
extension BFRecordScreenController: UICollectionViewDelegate, UICollectionViewDataSource, UIScrollViewDelegate {
    public func collectionView(_: UICollectionView, numberOfItemsInSection _: Int) -> Int {
        return itemModels.count
    }
    
    public func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let recordItem = itemModels[indexPath.item]
        var cell: BFImageCoverViewCell!
        if recordItem.mediaType == .VIDEO {
            cell = BFVideoCoverViewCell.gpuVideoViewCell(collectionView: collectionView, indexPath: indexPath)
        } else {
            cell = BFImageCoverViewCell.gpuImageViewCell(collectionView: collectionView, indexPath: indexPath)
        }
        recordItem.fetchCoverImgCallBack = { [weak self, weak cell, weak recordItem] _ in
            guard let sself = self, let item = recordItem else {
                return
            }
            cell?.addData()
            if item.mediaType == .IMAGE {
                sself.progressThumV.recordItem = item
                sself.progressThumV.isHidden = false
            }
        }
        recordItem.fetchPlayItemCallBack = { [weak self] item in
            guard let sself = self else {
                return
            }
            if indexPath.item == sself.currItemModelIndex {
                sself.reloadMaterial()
            }
        }
        cell.btnClickHandle = { [weak self] sender, _ in
            self?.playVideo(btn: sender)
        }
        cell.recordItem = recordItem
        return cell
    }
    
    public func collectionView(_: UICollectionView, didSelectItemAt _: IndexPath) {}
    
    public func scrollViewDidScroll(_ scrollView: UIScrollView) {
        let page = Int((scrollView.contentOffset.x + scrollView.frame.width / 2) / scrollView.frame.width)
        if page != currItemModelIndex {
            // 暂停
            pause()
            events.append(WithDrawModel(type: 0, timestamp: currentAssetProgress.seconds))
            // 重设撤销栈
            itemModels[currItemModelIndex].events = events
            currItemModelIndex = page
            
            events = itemModels[currItemModelIndex].events
            
            let recordItem = itemModels[currItemModelIndex]
            // 重绘录音区域
            progressThumV.recordItem = recordItem
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {[weak self] in
                self?.indirectionView?.resetAllSubViews(items: recordItem.voiceStickers, percenWidth: recordItem.mediaType == .IMAGE ? (self?.progressThumV.thumbImageWidth ?? 0) / 2.0 : 0, totalDuration: recordItem.materialDuraion)
            }
            // 更新缩略图
            progressThumV.isHidden = false
            progreddL.text = "00:00"
            // 重置指针
            currentAssetProgress = CMTime(seconds: 0, preferredTimescale: 1000)
            searchStopAtRecordRange()
            // 重置播放器
            assetPlayer?.seek(to: CMTime.zero)
            recordPlayer?.seek(to: CMTime.zero)
            
            searchStopAtRecordRange()
            changeWithDrawBtnLayout(false)
            pauseTime = 0
            if recordItem.mediaType == .VIDEO {
                reloadMaterial()
                assetPlayer?.seek(to: .zero, toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000))
            }
            if changeItemHandle != nil {
                changeItemHandle!(page)
            }
            
        }
    }
    
    public func updateContentOffset(index: Int) {
        let offx = collectionView.contentOffset.x
        if offx != collectionView.frame.width * CGFloat(index) {
            collectionView.setContentOffset(CGPoint(x: collectionView.frame.width * CGFloat(index), y: 0), animated: true)
        }
    }
}

// MARK: - 处理图片素材

/// 处理图片素材
public extension BFRecordScreenController {
    /// 处理图片音频的播放
    func imageRecordPlay() {
        if itemModels[currItemModelIndex].mediaType == .IMAGE {
            if itemModels[currItemModelIndex].materialDuraion <= 0 || currentAssetProgress.seconds >= itemModels[currItemModelIndex].materialDuraion {
                (collectionView.cellForItem(at: IndexPath(item: currItemModelIndex, section: 0)) as? BFImageCoverViewCell)?.playBtn.isSelected = false
                return
            }
            isNormalPlaying = true
            // 当开始播放时重置录音播放起始时间
            recordStartPlayTime = currentAssetProgress
            currenStartPlayTime = CMTime.zero
            playRecord(at: currentAssetProgress, periodicTimeObserver: { [weak self] currentT, currentItem in
                BFLog(1, message: "播放第段进度：\(currentT),\(currentItem)")
                self?.imageRecordProgress(progress: CMTimeGetSeconds(currentT))
            }, didPlayToEndTime: { [weak self] recordInfo, currentItem in
                BFLog(message: "播放第段结束：\(String(describing: recordInfo?.1)),\(String(describing: currentItem))")
                if (self?.itemModels[self?.currItemModelIndex ?? 0].voiceStickers.count ?? 0) <= ((recordInfo?.0 ?? 0) + 1) || (recordInfo?.1.endTime ?? 0) >= (self?.itemModels[self?.currItemModelIndex ?? 0].voiceStickers.last?.endTime ?? 0) {
                    self?.isEndPlay = true
                    self?.pause()
                    // 注：矫正进度--播放结束后当前指针应该到当前素材总时长
                    self?.currentAssetProgress = CMTime(seconds: self?.itemModels[self?.currItemModelIndex ?? 0].materialDuraion ?? 0, preferredTimescale: 1000)
                    self?.resetCurrentProgress()
                } else {
                    // 注：矫正进度--一段录音播放结束后当前指针应该到当前录音结束点
                    self?.currentAssetProgress = CMTime(seconds: self?.itemModels[self?.currItemModelIndex ?? 0].voiceStickers[(recordInfo?.0 ?? 0) + 1].startTime ?? 0, preferredTimescale: 1000)
                    self?.imageRecordPlay()
                }
            }) { [weak self] _, _ in
                DispatchQueue.main.asyncAfter(deadline: DispatchTime.now() + 3) {
                    self?.imageRecordPlay()
                }
            }
        }
    }
    
    /// 处理图片素材录音
    func imageRecordProgress(isRecord: Bool = false, progress: Float64) {
        if isRecord {
            currentAssetProgress = CMTime(seconds: itemModels[currItemModelIndex].materialDuraion + progress, preferredTimescale: 1000)
        } else {
            currentAssetProgress = CMTime(seconds: recordStartPlayTime.seconds + progress, preferredTimescale: 1000)
        }
        BFLog(1, message: "图片录音进度：\(progress),currentAssetProgress=\(currentAssetProgress),\(itemModels[currItemModelIndex].materialDuraion)")
        if itemModels[currItemModelIndex].mediaType == .IMAGE {
            /// 重置进度
            resetCurrentProgress()
        }
    }
    
    /// 重置进度
    func resetCurrentProgress() {
        DispatchQueue.main.async { [weak self] in
            self?.progreddL.text = String(format: "%@", (self?.currentAssetProgress.seconds ?? 0).formatDurationToHMS())
            self?.progressThumV.progress = (self?.currentAssetProgress.seconds ?? 0)
            self?.updateSubtitle(time: self?.currentAssetProgress ?? CMTime.zero)
        }
    }
}
