//
//  BFAudioSettingView.swift
//  BFRecordScreenKit
//
//  Created by ak on 2021/12/15.
//  功能：设置视频素材的音量

import Foundation
import BFMediaKit
import BFCommonKit
import BFUIKit

typealias BFAudioSettingViewCallBack = (_ haveSpeak: Float,_ noHaveSpeak:Float) -> Void

class BFAudioSettingView: UIView {

    var callBack: BFAudioSettingViewCallBack?
    
    //操作板背景
    let backView = UIView.init()
    // 标题
    public lazy var haveSpeakLab: UILabel = {
        let haveSpeakLab = UILabel()
        haveSpeakLab.font = UIFont.boldSystemFont(ofSize: 16)
        haveSpeakLab.textAlignment = .left
        haveSpeakLab.text = "说话时 视频音量"
        haveSpeakLab.textColor = .white
        haveSpeakLab.backgroundColor = .clear
        return haveSpeakLab
    }()
    
    //有录音
    lazy var haveSpeakSlider: BFUISlider = {
        let haveSpeakSlider = BFUISlider()
        let thbImage = UIImage.moduleImage(named: BFConfig.shared.silderPinUsedImageName, moduleName: "BFFramework", isAssets: false)
        haveSpeakSlider.setMinimumTrackImage(thbImage, for: .normal)
        haveSpeakSlider.setMaximumTrackImage(thbImage, for: .normal)
        haveSpeakSlider.setThumbImage(thbImage, for: .highlighted)
        haveSpeakSlider.setThumbImage(thbImage, for: .normal)
        haveSpeakSlider.maximumTrackTintColor = UIColor.hexColor(hexadecimal: "#E6E8E8")
        haveSpeakSlider.minimumTrackTintColor = UIColor.hexColor(hexadecimal: BFConfig.shared.styleColor.rawValue)
        haveSpeakSlider.tag = 10
        haveSpeakSlider.unit = "%"
//        speechSlidView.addTarget(self, action: #selector(sliderTouchEnded(sender:)), for: .touchUpInside)
        haveSpeakSlider.maximumValue = 100
        haveSpeakSlider.minimumValue = 0
        haveSpeakSlider.isMovePoint = false
        haveSpeakSlider.valueTextColor = UIColor.hexColor(hexadecimal: BFConfig.shared.styleColor.rawValue)
        haveSpeakSlider.value = 0.0

        return haveSpeakSlider
    }()
    
    // 标题
    public lazy var noSpeakLab: UILabel = {
        let noSpeakLab = UILabel()
        noSpeakLab.font = UIFont.boldSystemFont(ofSize: 16)
        noSpeakLab.textAlignment = .left
        noSpeakLab.text = "没说话时 视频音量"
        noSpeakLab.textColor = .white
        return noSpeakLab
    }()
    
    lazy var noSpeakSlider: BFUISlider = {
        let noSpeakSlider = BFUISlider()
        let thbImage = UIImage.moduleImage(named: BFConfig.shared.silderPinUsedImageName, moduleName: "BFFramework", isAssets: false)
        noSpeakSlider.setMinimumTrackImage(thbImage, for: .normal)
        noSpeakSlider.setMaximumTrackImage(thbImage, for: .normal)
        noSpeakSlider.setThumbImage(thbImage, for: .highlighted)
        noSpeakSlider.setThumbImage(thbImage, for: .normal)
        noSpeakSlider.maximumTrackTintColor = UIColor.hexColor(hexadecimal: "#E6E8E8")
        noSpeakSlider.minimumTrackTintColor = UIColor.hexColor(hexadecimal: BFConfig.shared.styleColor.rawValue)
        noSpeakSlider.tag = 20
        noSpeakSlider.isMovePoint = false
//        speechSlidView.addTarget(self, action: #selector(sliderTouchEnded(sender:)), for: .touchUpInside)
        noSpeakSlider.maximumValue = 100
        noSpeakSlider.minimumValue = 0
        noSpeakSlider.unit = "%"
        noSpeakSlider.valueTextColor = UIColor.hexColor(hexadecimal: BFConfig.shared.styleColor.rawValue)
        noSpeakSlider.value = 100

        return noSpeakSlider
    }()

    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.backgroundColor = UIColor.clear
        addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(hidden(tap:))))
        
       
        backView.backgroundColor = .black
        addSubview(backView)
        backView.snp.makeConstraints { make in
            make.right.equalTo(self.snp.right)
            make.bottom.equalTo(self.snp.bottom)
            make.width.equalTo(cScreenWidth)
            make.height.equalTo(220)
        }
 
        
        backView.addSubview(haveSpeakSlider)
        backView.addSubview(noSpeakSlider)
        backView.addSubview(haveSpeakLab)
        backView.addSubview(noSpeakLab)
        
        
        haveSpeakLab.snp.makeConstraints { make in
            make.left.equalToSuperview().offset(16)
            make.right.equalToSuperview().offset(-16)
            make.top.equalToSuperview().offset(25)
        }
        haveSpeakSlider.snp.makeConstraints { make in
            make.left.equalToSuperview().offset(16)
            make.right.equalToSuperview().offset(-16)
            make.top.equalTo(haveSpeakLab).offset(24)
        }
        
        noSpeakLab.snp.makeConstraints { make in
            make.left.equalToSuperview().offset(16)
            make.right.equalToSuperview().offset(-16)
            make.top.equalTo(haveSpeakSlider).offset(42)
        }
        noSpeakSlider.snp.makeConstraints { make in
            make.left.equalToSuperview().offset(16)
            make.right.equalToSuperview().offset(-16)
            make.top.equalTo(noSpeakLab).offset(24)
        }
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        self.backView.addCorner(roundingCorners: [.topLeft, .topRight], corner: 10)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    @objc func hidden(tap:UITapGestureRecognizer){
        if !backView.bounds.contains(tap.location(in: backView)){
            self.isHidden = true
            if(callBack != nil){
                callBack!(haveSpeakSlider.value,noSpeakSlider.value)
            }
        }
    }
    
}

